@@ -23,7 +23,6 @@
 
 #include <Lum/OS/Font.h>
 
-#include <iostream>
 Compass::Compass()
  : arrowColor(0.0,0.0,0.0,Lum::OS::Display::blackColor),
    arrowShadowColor(0.6,0.6,0.6,Lum::OS::Display::whiteColor)
@@ -193,3 +192,15 @@ void Compass::Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
   Control::Resync(model,msg);
 }
 
+Compass* Compass::Create(Lum::Base::Model* model,
+                         bool horizontalFlex,
+                         bool verticalFlex)
+{
+  Compass *compass=new Compass();
+
+  compass->SetFlex(horizontalFlex,verticalFlex);
+  compass->SetModel(model);
+
+  return compass;
+}
+
