@@ -42,8 +42,6 @@ extern "C" {
 #include <netinet/in.h>
 
 #include <fstream>
-#include <iomanip>
-#include <sstream>
 
 #include <Lum/Def/Menu.h>
 #include <Lum/Def/MultiView.h>
@@ -51,6 +49,8 @@ extern "C" {
 #include <Lum/OS/Probe.h>
 #include <Lum/OS/Thread.h>
 
+#include <Lum/OS/Manager/Repository.h>
+
 #include <Lum/Base/DateTime.h>
 #include <Lum/Base/L10N.h>
 #include <Lum/Base/Path.h>
@@ -82,7 +82,9 @@ extern "C" {
 #include <Lum/View.h>
 
 #include "Compass.h"
+#include "Speed.h"
 #include "TimeLine.h"
+#include "Util.h"
 
 #include <iostream>
 
@@ -102,6 +104,44 @@ static long reconnectTime   = 10; // Period of time after which we check for dis
                                   // trigger reconnection
 static long refreshTime     = 10; // Period of time after which we update the stream views
 
+std::wstring SystemTimeToISO8601(const Lum::Base::SystemTime& time)
+{
+  Lum::Base::DateTime t;
+  std::wstring        res;
+
+  res.reserve(10+1+8+1);
+
+  time.GetUTCDateTime(t);
+
+  // Date
+  res.append(1,(wchar_t)(L'0'+(t.year / 1000) % 10));
+  res.append(1,(wchar_t)(L'0'+(t.year /  100) % 10));
+  res.append(1,(wchar_t)(L'0'+(t.year /   10) % 10));
+  res.append(1,(wchar_t)(L'0'+(t.year /    1) % 10));
+  res.append(L"-");
+  res.append(1,(wchar_t)(L'0'+(t.month / 10) % 10));
+  res.append(1,(wchar_t)(L'0'+(t.month /  1) % 10));
+  res.append(L"-");
+  res.append(1,(wchar_t)(L'0'+(t.dayOfMonth / 10) % 10));
+  res.append(1,(wchar_t)(L'0'+(t.dayOfMonth /  1) % 10));
+
+  res.append(1,'T');
+
+  // Time
+  res.append(1,(wchar_t)(L'0'+(t.hour /   10) % 10));
+  res.append(1,(wchar_t)(L'0'+(t.hour /    1) % 10));
+  res.append(L":");
+  res.append(1,(wchar_t)(L'0'+(t.minute / 10) % 10));
+  res.append(1,(wchar_t)(L'0'+(t.minute /  1) % 10));
+  res.append(L":");
+  res.append(1,(wchar_t)(L'0'+(t.second / 10) % 10));
+  res.append(1,(wchar_t)(L'0'+(t.second /  1) % 10));
+
+  res.append(1,'Z');
+
+  return res;
+}
+
 class GPSData
 {
 public:
@@ -659,25 +699,6 @@ static void gps_location_stopped(LocationGPSDControl *control, gpointer userdata
 static void gps_location_error(LocationGPSDControl *control, gpointer userdata);
 #endif
 
-std::wstring DoubleToString(double value)
-{
-  if (!isnan(value)) {
-    std::stringstream buffer;
-
-    buffer.imbue(std::locale(""));
-    buffer << std::fixed;
-    buffer << std::showpoint;
-    buffer.precision(2);
-
-    buffer << value;
-
-    return Lum::Base::StringToWString(buffer.str());
-  }
-  else {
-    return L"";
-  }
-}
-
 class Track
 {
 private:
@@ -858,6 +879,7 @@ private:
   Lum::Model::ActionRef    reconnectCheckTimer;
 
   Lum::Model::BooleanRef   tracking;
+  Lum::Model::BooleanRef   prohibitScreenBlanking;
 
   std::ofstream            trackFile;
 
@@ -1047,7 +1069,8 @@ public:
     dataChangedAction(new Lum::Model::Action()),
     refreshTimer(new Lum::Model::Action()),
     reconnectCheckTimer(new Lum::Model::Action()),
-    tracking(new Lum::Model::Boolean(false))
+    tracking(new Lum::Model::Boolean(false)),
+    prohibitScreenBlanking(new Lum::Model::Boolean(false))
 #if defined(HAVE_LIB_LOCATION)
     ,device(NULL)
     ,control(NULL)
@@ -1076,6 +1099,12 @@ public:
     Observe(reconnectCheckTimer);
     Observe(trackSelection);
 
+    if (Lum::OS::Manager::repository->GetDisplayManager()==NULL) {
+      prohibitScreenBlanking->Disable();
+    }
+
+    Observe(prohibitScreenBlanking);
+
     ::window=this;
 
     startTrackingAction->Enable();
@@ -1110,7 +1139,6 @@ public:
     Lum::Label            *label;
     Lum::Panel            *hPanel;
     Lum::Table            *table;
-    Compass               *compass;
 
     Lum::Def::MultiView multiView(Lum::Def::Desc(L"GPSJinni"));
 
@@ -1152,15 +1180,15 @@ public:
                                                 Lum::Def::Desc(L"Satellites"),
                                                 table));
 
-    compass=new Compass();
-    compass->SetFlex(true,true);
-    compass->SetModel(direction);
-
     multiView.AddView(Lum::Def::MultiView::View(2,
-                                                Lum::Def::Desc(L"Compass"),
-                                                compass));
+                                                Lum::Def::Desc(L"Speed"),
+                                                Speed::Create(direction,true,true)));
 
     multiView.AddView(Lum::Def::MultiView::View(3,
+                                                Lum::Def::Desc(L"Compass"),
+                                                Compass::Create(direction,true,true)));
+
+    multiView.AddView(Lum::Def::MultiView::View(4,
                                                 Lum::Def::Desc(L"Diagrams"),
                                                 Lum::VPanel::Create(true,true)
                                                 ->Add(Lum::Text::Create(L"Speed (m/s)"))
@@ -1182,7 +1210,7 @@ public:
     table->SetSelection(trackSelection);
     table->GetTableView()->SetAutoFitColumns(true);
 
-    multiView.AddView(Lum::Def::MultiView::View(4,
+    multiView.AddView(Lum::Def::MultiView::View(5,
                                                 Lum::Def::Desc(L"Tracking"),
                                                 Lum::VPanel::Create(true,true)
                                                 ->Add(Lum::HPanel::Create(true,false)
@@ -1213,6 +1241,10 @@ public:
     preMenu
       ->GroupProject()
         ->ActionQuit(GetClosedAction())
+      ->End()
+      ->GroupEdit()
+        ->Boolean(Lum::Def::Boolean(Lum::Def::Desc(L"Prohibit screen blanking"),
+                                    prohibitScreenBlanking))
       ->End();
 
     Lum::Def::Menu *postMenu=Lum::Def::Menu::Create();
@@ -1427,6 +1459,14 @@ public:
     tracks->On();
   }
 
+  struct DataPoint
+  {
+    Lum::Base::SystemTime time;
+    double                lat;
+    double                lon;
+    double                alt;
+  };
+
   void ExportTrack(const std::wstring& name)
   {
     Lum::Base::Path                path;
@@ -1540,26 +1580,7 @@ public:
       return;
     }
 
-    out << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << std::endl;
-    out << "<kml xmlns=\"http://www.opengis.net/kml/2.2\">" << std::endl;
-    out << "  <Document>" << std::endl;
-    out << "    <name>" << Lum::Base::WStringToUTF8(name) << ".kml" << "</name>" << std::endl;
-    out << "    <open>1</open>" << std::endl;
-    out << "    <Style id=\"lineStyle\">" << std::endl;
-    out << "      <LineStyle>" << std::endl;
-    out << "        <color>7f0000ff</color>" << std::endl;
-    out << "        <width>4</width>" << std::endl;
-    out << "      </LineStyle>" << std::endl;
-    out << "    </Style>" << std::endl;
-
-    out << "    <Placemark>" << std::endl;
-    out << "      <name>" << Lum::Base::WStringToUTF8(name) << "</name>" << std::endl;
-    out << "      <styleUrl>#lineStyle</styleUrl>" << std::endl;
-    out << "      <LineString>" << std::endl;
-    out << "        <extrude>0</extrude>" << std::endl;
-//    out << "        <tessellate>1</tessellate>" << std::endl;
-    out << "        <altitudeMode>absolute</altitudeMode>" << std::endl;
-    out << "        <coordinates>" << std::endl;
+    std::list<DataPoint> data;
 
     while (in) {
       unsigned long flon;
@@ -1573,10 +1594,7 @@ public:
       in.read((char*)&ftim,sizeof(ftim));
 
       if (in) {
-        double                lon;
-        double                lat;
-        double                alt;
-        Lum::Base::SystemTime time;
+        DataPoint dataPoint;
 
         flon=ntohl(flon);
         flat=ntohl(flat);
@@ -1584,17 +1602,158 @@ public:
         ftim=ntohl(ftim);
 
         if (tfv==1) {
-          lon=flon/10000000.0-90.0;
-          lat=flat/10000000.0-180.0;
+          dataPoint.lon=flon/10000000.0-90.0;
+          dataPoint.lat=flat/10000000.0-180.0;
         }
         else {
-          lon=flon/10000000.0-180.0;
-          lat=flat/10000000.0-90.0;
+          dataPoint.lon=flon/10000000.0-180.0;
+          dataPoint.lat=flat/10000000.0-90.0;
         }
-        alt=falt/100;
-        time.SetTime(ftim);
+        dataPoint.alt=falt/100;
+        dataPoint.time.SetTime(ftim);
 
-        out << "          " << lon << "," << lat << "," << alt << std::endl;
+        data.push_back(dataPoint);
+      }
+    }
+
+    out << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << std::endl;
+    out << "<kml xmlns=\"http://www.opengis.net/kml/2.2\">" << std::endl;
+    out << "  <Document>" << std::endl;
+    out << "    <name>" << Lum::Base::WStringToUTF8(name) << ".kml" << "</name>" << std::endl;
+    out << "    <open>1</open>" << std::endl;
+    out << std::endl;
+    out << "    <Style id=\"startpointStyle\">" << std::endl;
+    out << "      <IconStyle>" << std::endl;
+    out << "        <Icon>" << std::endl;
+    out << "          <href>http://maps.google.com/mapfiles/kml/paddle/A.png</href>" << std::endl;
+    out << "        </Icon>" << std::endl;
+    out << "      </IconStyle>" << std::endl;
+    out << "    </Style>" << std::endl;
+    out << std::endl;
+    out << "    <Style id=\"waypointStyle\">" << std::endl;
+    out << "      <IconStyle>" << std::endl;
+    out << "        <Icon>" << std::endl;
+    out << "          <href>http://maps.google.com/mapfiles/kml/pushpin/ylw-pushpin.png</href>" << std::endl;
+    out << "        </Icon>" << std::endl;
+    out << "      </IconStyle>" << std::endl;
+    out << "    </Style>" << std::endl;
+    out << std::endl;
+    out << "    <Style id=\"endpointStyle\">" << std::endl;
+    out << "      <IconStyle>" << std::endl;
+    out << "        <Icon>" << std::endl;
+    out << "          <href>http://maps.google.com/mapfiles/kml/paddle/B.png</href>" << std::endl;
+    out << "        </Icon>" << std::endl;
+    out << "      </IconStyle>" << std::endl;
+    out << "    </Style>" << std::endl;
+    out << std::endl;
+
+    out << "    <Folder>" << std::endl;
+    out << "      <name>" << Lum::Base::WStringToUTF8(name) << " waypoints with timestamps</name>" << std::endl;
+    out << "      <open>1</open>" << std::endl;
+    out << std::endl;
+    out << "      <Style>" << std::endl;
+    out << "        <ListStyle>" << std::endl;
+    out << "          <listItemType>checkHideChildren</listItemType>" << std::endl;
+    out << "        </ListStyle>" << std::endl;
+    out << "      </Style>" << std::endl;
+
+    for (std::list<DataPoint>::const_iterator dataPoint=data.begin();
+         dataPoint!=data.end();
+         ++dataPoint) {
+      out << std::endl;
+      out << "      <Placemark>" << std::endl;
+      out << "        <TimeStamp>" << std::endl;
+      out << "          <when>" << Lum::Base::WStringToString(SystemTimeToISO8601(dataPoint->time)) << "</when>" << std::endl;
+      out << "        </TimeStamp>" << std::endl;
+      out << "        <styleUrl>#waypointStyle</styleUrl>" << std::endl;
+      out << "        <Point>" << std::endl;
+      out << "          <coordinates>" << dataPoint->lon << "," << dataPoint->lat << "," << dataPoint->alt << "</coordinates>" << std::endl;
+      out << "        </Point>" << std::endl;
+      out << "      </Placemark>" << std::endl;
+    }
+    out << "    </Folder>" << std::endl;
+
+    out << std::endl;
+    out << std::endl;
+
+    out << "    <Folder>" << std::endl;
+    out << "      <name>" << Lum::Base::WStringToUTF8(name) << " path waypoints</name>" << std::endl;
+    out << "      <open>1</open>" << std::endl;
+    out << std::endl;
+    out << "      <Style>" << std::endl;
+    out << "        <ListStyle>" << std::endl;
+    out << "          <listItemType>checkHideChildren</listItemType>" << std::endl;
+    out << "        </ListStyle>" << std::endl;
+    out << "      </Style>" << std::endl;
+
+    for (std::list<DataPoint>::const_iterator dataPoint=data.begin();
+         dataPoint!=data.end();
+         ++dataPoint) {
+      out << std::endl;
+      out << "      <Placemark>" << std::endl;
+      out << "        <styleUrl>#waypointStyle</styleUrl>" << std::endl;
+      out << "        <Point>" << std::endl;
+      out << "          <coordinates>" << dataPoint->lon << "," << dataPoint->lat << "," << dataPoint->alt << "</coordinates>" << std::endl;
+      out << "        </Point>" << std::endl;
+      out << "      </Placemark>" << std::endl;
+    }
+    out << "    </Folder>" << std::endl;
+
+    out << std::endl;
+    out << std::endl;
+
+    if (data.size()>0) {
+      out << "    <Placemark>" << std::endl;
+      out << "      <styleUrl>#startpointStyle</styleUrl>" << std::endl;
+      out << "      <name>Start</name>" << std::endl;
+      out << "      <Point>" << std::endl;
+      out << "        <coordinates>" << data.begin()->lon << "," << data.begin()->lat << "," << data.begin()->alt << "</coordinates>" << std::endl;
+      out << "      </Point>" << std::endl;
+      out << "    </Placemark>" << std::endl;
+      out << std::endl;
+    }
+
+    if (data.size()>0) {
+      out << "    <Placemark>" << std::endl;
+      out << "      <styleUrl>#endpointStyle</styleUrl>" << std::endl;
+      out << "      <name>End</name>" << std::endl;
+      out << "      <Point>" << std::endl;
+      out << "        <coordinates>" << data.rbegin()->lon << "," << data.rbegin()->lat << "," << data.rbegin()->alt << "</coordinates>" << std::endl;
+      out << "      </Point>" << std::endl;
+      out << "    </Placemark>" << std::endl;
+      out << std::endl;
+    }
+
+    out << "    <Placemark>" << std::endl;
+    out << "      <name>" << Lum::Base::WStringToUTF8(name) << " path</name>" << std::endl;
+    out << std::endl;
+    out << "      <Style>" << std::endl;
+    out << "        <LineStyle>" << std::endl;
+    out << "          <color>ff0000ff</color>" << std::endl;
+    out << "          <width>2</width>" << std::endl;
+    out << "        </LineStyle>" << std::endl;
+    out << "      </Style>" << std::endl;
+    out << std::endl;
+    out << "      <LineString>" << std::endl;
+    //out << "        <extrude>0</extrude>" << std::endl;
+    out << "        <tessellate>1</tessellate>" << std::endl;
+    out << "        <altitudeMode>clampToGround</altitudeMode>" << std::endl;
+    out << "        <coordinates>" << std::endl;
+
+    double lastLat=1000;
+    double lastLon=1000;
+    double lastAlt=1000;
+
+    for (std::list<DataPoint>::const_iterator dataPoint=data.begin();
+         dataPoint!=data.end();
+         ++dataPoint) {
+      if (dataPoint->lon!=lastLon ||
+          dataPoint->lat!=lastLat ||
+          dataPoint->alt!=lastAlt) {
+        out << "          " << dataPoint->lon << "," << dataPoint->lat << "," << dataPoint->alt << std::endl;
+        lastLon=dataPoint->lon;
+        lastLat=dataPoint->lat;
+        lastAlt=dataPoint->alt;
       }
     }
 
@@ -1916,6 +2075,13 @@ public:
         exportTrackAction->Disable();
       }
     }
+    else if (model==prohibitScreenBlanking) {
+      Lum::OS::Manager::DisplayManager *displayManager=Lum::OS::Manager::repository->GetDisplayManager();
+
+      if (displayManager!=NULL) {
+        displayManager->AllowScreenBlanking(!prohibitScreenBlanking->Get());
+      }
+    }
 
     Dialog::Resync(model,msg);
   }
